import java.util.Arrays;     // sort()
import java.util.Comparator; // idem

public class quipe
{
   //
   // Valeurs par dfaut
   //
   private static final int
      NB_JOUEURS_DFAUT = 10;
   private static final String
      NOM_DFAUT = "Propritaire inconnu";

   //
   // Attributs
   //
   private Joueur [] m_Joueurs;
   private String m_Nom;
   private int m_NbJoueurs;

   //
   // Accesseurs/ manipulateurs pour le nom
   //
   public String GetNom ()
   {
      return m_Nom;
   }
   public void SetNom (final String Nom)
      throws Exception
   {
      if (Nom == null)
         throw new Exception ("Nom ne peut tre nul");
      SetNomBrut (Nom);
   }
   private void SetNomBrut (final String Nom)
   {
      m_Nom = Nom;
   }

   //
   // Constructeurs
   //
   public quipe ()
   {
      SetNomBrut (NOM_DFAUT);
      CrerquipeBrut (NB_JOUEURS_DFAUT);
   }
   public quipe (final String Nom)
      throws Exception
   {
      SetNom (Nom);
      CrerquipeBrut (NB_JOUEURS_DFAUT);
      m_NbJoueurs = 0;
   }
   public quipe (final String Nom, final int NbJoueurs)
      throws Exception
   {
      SetNom (Nom);
      Crerquipe (NbJoueurs);
      m_NbJoueurs = 0;
   }
   public quipe (final quipe Original)
   {
      copy (Original);
   }

   //
   // Comparaison et affectation
   //
   public void copy (final quipe Original)
   {
      SetNomBrut (Original.GetNom ());
      m_Joueurs = new Joueur[Original.m_Joueurs.length];
      for (int i = 0; i < m_Joueurs.length; i++)
      {
         if (Original.m_Joueurs[i] != null)
            m_Joueurs[i] = new Joueur (Original.m_Joueurs[i]);
         else
            m_Joueurs[i] = null;
      }
      m_NbJoueurs = Original.m_NbJoueurs;
   }
   public boolean equals (final quipe Autre) // politique arbitraire
   {
      return Autre != null &&
             GetNom ().equals (Autre.GetNom ());
   }

   //
   // Utilitaires de gestion interne
   //
   private void Crerquipe (final int NbJoueurs)
      throws Exception
   {
      if (NbJoueurs <= 0)
         throw new Exception ("Nb joueurs invalide: " + NbJoueurs);
      CrerquipeBrut (NbJoueurs);
   }
   private void CrerquipeBrut (final int NbJoueurs)
   {
      m_Joueurs = new Joueur[NbJoueurs];
      for (int i = 0; i < m_Joueurs.length; i++)
         m_Joueurs[i] = null;
   }

   //
   // Utilitaires accessibles  l'externe
   //
   public boolean Pleine ()
   {
      return m_NbJoueurs == m_Joueurs.length;
   }
   public boolean Vide ()
   {
      return m_NbJoueurs == 0;
   }
   public static class JoueurNulException extends Exception
   {
      public JoueurNulException ()
      {
         super ("Joueur nul");
      }
   }
   public static class JoueurExisteException extends Exception
   {
      public JoueurExisteException (final String Nom)
      {
         super ("Joueur " + Nom + " existe");
      }
   }
   public static class quipePleineException extends Exception
   {
      public quipePleineException ()
      {
         super ("quipe pleine");
      }
   }
   public boolean JoueurExiste (final Joueur j)
      throws JoueurNulException
   {
      if (j == null)
         throw new JoueurNulException ();
      for (int i = 0; i < m_Joueurs.length; i++)
         if (m_Joueurs[i] != null && m_Joueurs[i].equals (j))
            return true;
      return false;
   }
   public void AjouterJoueur (final Joueur j)
      throws JoueurNulException, JoueurExisteException, quipePleineException
   {
      if (j == null)
         throw new JoueurNulException ();
      if (Pleine ())
         throw new quipePleineException ();
      if (JoueurExiste (j))
         throw new JoueurExisteException (j.GetNom ());
      int i = 0;
      while (i < m_Joueurs.length && m_Joueurs[i] != null)
         i++;
      m_Joueurs[i] = new Joueur (j);
      m_NbJoueurs ++;
   }
   //
   // Utilitaires statistiques
   //
   private void TrierSelonPoints (final Joueur [] Liste)
   {
      //
      // Un tri  bulles aurait t trs correct, mais j'ai
      // pris la libert d'utiliser un tri standard
      //
      Arrays.sort
         (Liste, new Comparator ()
                 {
                    public int compare (Object j0, Object j1)
                    {
                       int Rsultat;
                       if (j0 == null && j1 == null)
                          Rsultat = 0;
                       else if (j0 == null)
                          return 1;
                       else if (j1 == null)
                          return -1;
                       else
                          Rsultat = ((Joueur) j1).GetPoints () - ((Joueur) j0).GetPoints ();
                       return Rsultat;
                    }
                 }
         );

   }
   public Joueur [] GetListePointeurs ()
   {
      Joueur [] Liste = new Joueur[m_Joueurs.length];
      for (int i = 0; i < m_Joueurs.length; i++)
      {
         if (m_Joueurs[i] != null)
            Liste[i] = new Joueur (m_Joueurs[i]);
         else
            Liste[i] = null;
      }
      //
      // Trier la liste
      //
      TrierSelonPoints (Liste);
      return Liste;
   }

   //
   // Simple, mais pourrait tre plus efficace
   //
   public Joueur GetMeilleurPointeur ()
   {
      return GetListePointeurs()[0];
   }
   public Joueur GetPirePointeur ()
   {
      Joueur [] Temp = GetListePointeurs();
      for (int i = Temp.length; --i >= 0;)
         if (Temp[i] != null)
            return Temp[i];
      return null;
   }
}